//Accel library
#include <AccelStepper.h>

//Stepper definieren (oder deklarieren?)
AccelStepper stepper1(AccelStepper::FULL2WIRE, 4, 5); // positiv -  vorwärts (Rechts)
AccelStepper stepper2(AccelStepper::FULL2WIRE, 2, 3); // positiv - rückwärts (Links)
//Hinweise:
//Pin 2 und Pin 4 sind die Step-pins
//Pin 3 und Pin 5 die Richtungs-Pins

//Farbsensor Pins
#define S0 10
#define S1 9
#define S2 8
#define S3 7
#define sensorOut 6

//Ultraschall Pins
//vorne
const int trig1Pin = 11;
const int echo1Pin = 12;
//hinten
const int trig2Pin = 18;
const int echo2Pin = 13;

void setup() {
  // Farbsensor Pins
  pinMode(S0, OUTPUT);
  pinMode(S1, OUTPUT);
  pinMode(S2, OUTPUT);
  pinMode(S3, OUTPUT);
  pinMode(sensorOut, INPUT);
  digitalWrite(S0, HIGH);
  digitalWrite(S1, HIGH);

  //Ultraschall-Abstandsmesser(USM) Pins
  //Vorne:
  pinMode(trig1Pin, OUTPUT);
  pinMode(echo1Pin, INPUT);
  //Hinten:
  pinMode(trig2Pin, OUTPUT);
  pinMode(echo2Pin, INPUT);

  //Stepper
  //Hinweise:
  //setMaxSpeed sollte bei allen Steppern der Einfachheit halber immer das gleiche Vorzeichen an der Zahl haben
  //bei setSpeed kann man im setup schon variieren, allerdings sind gleiche Vorzeichen auch hier zu empfehlen, wenn man später im loop den Motor in verschiedene Richtungen drehen lassen möchte
  //Die Richtung in die sich die Räder an den Motoren drehen lässt sich durch eine Vorzeichenänderung bei setSpeed und/oder setMaxSpeed umkehren
  //rechts
  stepper1.setMaxSpeed(2000.0);
  stepper1.setSpeed(1000.0);// Schritte pro Minute
  //links
  stepper2.setMaxSpeed(2000.0);
  stepper2.setSpeed(1000.0); // Schritte pro Minute

  //Stepper, Blink
  pinMode(2, OUTPUT);
  pinMode(3, OUTPUT);
  pinMode(4, OUTPUT);
  pinMode(5, OUTPUT);

  Serial.begin(115200);
}

unsigned long VorhZeit = 0; // Zeitpunkt der letzten Messung (vorh = vorherige)
unsigned long AktuelleZeit = 0; // Anzahl der Millisekunden seit Beginn des Programms, bis zum Beginn der loop-Funktion

const long Intervall1 = 100;
const long Intervall2 = 10;

//Anfangswerte für den jeweilgen Abstand zur Wand
long distVorn = 15;
long distHinten = 15;


void loop() {
  int rot = Rot();
  int blau = Blau();

  AktuelleZeit = millis();
  if (AktuelleZeit - VorhZeit >= Intervall1) {  // Aktueller Zeitpunkt - Zeitpunkt der letzten Messung >= Messinterval, wenn wahr -> wird neu gemessen
    VorhZeit = AktuelleZeit;
    distVorn = Vorn();
    distHinten = Hinten();
  }

  long Korrektur = distVorn - distHinten; // Korrektur = differenz der beiden Messungen vorne und hinten
  //Fahrtrichtung geradeaus, ohne Korrektur, so lange wie der Abstand links größer als 30cm ist
  stepper1.setSpeed(-300);
  stepper2.setSpeed(300);
  if (distVorn < 30 && distHinten < 30) {
    stepper1.setSpeed(-300 - (Korrektur * 20)); //Berechnung der neuen Geschwindigkeit, unter Vorzeichenberückichtigung.
    stepper2.setSpeed(300 - (Korrektur * 20));
    //Faktor 20 ist zur Verstärkung des Korrekturfaktors, da beispielsweise bei einer Differenz von 2cm,
    //ohne  diesen zusätzlichen Faktor, die Korrektur quasi nichtig wäre
  }
  stepper1.runSpeed();
  stepper2.runSpeed();

  //Farben
  if (rot <= 10) {
    for (int i = 0; i < 135; i++) {
      SchrittRechtsRuckwarts();
      SchrittLinksVorwarts();
    }
  }
  if (blau <= 13) {
    for (int i = 0; i < 135; i++) {
      SchrittRechtsVorwarts();
      SchrittLinksRuckwarts();
    }
  }
}

//Funktionen

//Ultraschallsensoren (Abstandsmesser)
double Vorn() {
  long Zeit, Entf;
  digitalWrite(trig1Pin, LOW);
  delayMicroseconds(2);
  digitalWrite(trig1Pin, HIGH);// Trigger sendet 10 Microsekunden ein Signal aus
  delayMicroseconds(10);
  digitalWrite(trig1Pin, LOW);
  Zeit = pulseIn(echo1Pin, HIGH); // Echo empfängt refelktion und die Zeit, die der Impuls für den Hin - und Rückweg gebraucht hat wird ausgelesen
  Entf = Zeit / 29.1 / 2 ; // Umrchnung von Zeit in cm

  return Entf;
}

double Hinten() {
  long Zeit, Entf;
  digitalWrite(trig2Pin, LOW);
  delayMicroseconds(2);
  digitalWrite(trig2Pin, HIGH);// Trigger sendet 10 Microsekunden ein Signal aus
  delayMicroseconds(10);
  digitalWrite(trig2Pin, LOW);
  Zeit = pulseIn(echo2Pin, HIGH);
  Entf = Zeit / 29.1 / 2 ;

  return Entf;
}

// RGB-Sensor
int Rot() {
  digitalWrite(S2, LOW);
  digitalWrite(S3, LOW);
  int FrequenzROT = pulseIn(sensorOut, LOW);
  Serial.print("R= ");
  Serial.print(FrequenzROT);
  Serial.println("  ");
  return (FrequenzROT);
}

int Blau() {
  digitalWrite(S2, LOW);
  digitalWrite(S3, HIGH);
  int FrequenzBLAU = pulseIn(sensorOut, LOW);
  Serial.print("B= ");
  Serial.print(FrequenzBLAU);
  Serial.print("  ");
  Serial.print("\t");
  return (FrequenzBLAU);
}

// Blink Ansteuerung der Motoren
//Hinweise:
//2 und 4 sind Step Pins
//3 und 5 sind Richtungs pins
void SchrittRechtsVorwarts() {
  digitalWrite(4, HIGH);
  digitalWrite(5, HIGH);
  delay(5);
  digitalWrite(4, LOW);
  digitalWrite(5, LOW);
  delay(10);
}
void SchrittRechtsRuckwarts() {
  digitalWrite(4, HIGH);
  digitalWrite(5, LOW);
  delay(5);
  digitalWrite(5, HIGH);
  digitalWrite(4, LOW);
  delay(10);
}

void SchrittLinksVorwarts() {
  digitalWrite(2, HIGH);
  digitalWrite(3, LOW);
  delay(5);
  digitalWrite(2, LOW);
  digitalWrite(3, HIGH);
  delay(10);
}

void SchrittLinksRuckwarts() {
  digitalWrite(2, HIGH);
  digitalWrite(3, HIGH);
  delay(5);
  digitalWrite(2, LOW);
  digitalWrite(3, LOW);
  delay(10);
}

